import pygame

import defs, graphic, sprite

resize_surfaces = []

def init_missile():
    for size in defs.missile_sizes:
        new_surface = pygame.surface.Surface(size)
        new_surface.convert()
        new_surface.set_colorkey(defs.invisible_color)
        resize_surfaces.append(new_surface)

class Missile(object):
    def __init__(self, spritesheet, type, damage, (x, y, z), (tx, ty, tz)):
        self.spritesheet = spritesheet
        self.type   = type
        self.damage = damage
        (self.x,  self.y,  self.z)  = (x,  y,  z)
        (self.tx, self.ty, self.tz) = (tx, ty, tz)
        (self.dx, self.dy, self.dz) = (self.tx - self.x, self.ty - self.y, self.tz - self.z)
        self.delete_flag = False
    
    def can_delete(self):
        return self.delete_flag
    
    def update(self, delay, events, world):
        self.x += (self.dx / float(defs.z_difference))*delay
        self.y += (self.dy / float(defs.z_difference))*delay
        if self.dz > 0:
            self.z += delay
        elif self.dz < 0:
            self.z -= delay
        if self.z > defs.z_distance or self.z < 0:
            self.delete_flag = True
            
    def draw(self, surface, (dx, dy)):
        if self.delete_flag == True:
            return
        
        rect = pygame.rect.Rect(0, 0, 0, 0)
        
        scale = int((self.z * len(resize_surfaces)) // defs.z_distance) 
        if not (0 <= scale < len(resize_surfaces)):
            return
        s_size  = defs.missile_sizes[scale]
        s_image = resize_surfaces[scale]
        
        if self.dz > 0:
            pygame.transform.scale(self.spritesheet[0], s_size, s_image)
        else:
            pygame.transform.scale(self.spritesheet[1], s_size, s_image)
        rect.size = s_size
        
        # Draw the missile
        rect.center = (self.x, self.y)
        surface.blit(s_image, rect.topleft)

class Arrow(Missile):
    def __init__(self, (x, y, z), (tx, ty, tz)):
        image = graphic.load_image("arrow.png")
        spritesheet = sprite.SpriteSheet(image, (128, 128))
        Missile.__init__(self, spritesheet, "arrow", defs.arrow_damage, (x, y, z), (tx, ty, tz))
