from os.path import join

import pygame

import character
import defs

class World(object):
    def __init__(self, background, castle, player):
        self.background = background
        self.castle     = castle
        self.player     = player
        self.enemies    = []
        
        self.enemy_timer = 3000
        self.enemy_queue = [
            ("play", "track1.ogg", 500),
            "wait",
            (character.Archer((450, 150)), 1000),
            (character.Archer((400, 150)), 2000),
            ("play", "track2.ogg", 2500),
            "wait",
            (character.Archer((300, 150)), 3000),
            (character.StrongArcher((450, 150)), 3000),
            (character.Archer((600, 150)), 3000),
            ("play", "track3.ogg", 3500),
            "wait",
            (character.Archer((600, 150)), 4000),
            (character.StrongArcher((300, 150)), 4000),
            (character.QuickArcher((450, 150)), 4300),
            ("play", "track4.ogg", 4500),
            "wait",
            (character.EliteArcher((600, 150)), 5000),
            (character.EliteArcher((300, 150)), 5000),
            ("play", "track5.ogg", 5500),
            "wait",
            (character.KingArcher((450, 150)), 6000),
            ("play", "track6.ogg", 6500),
            "wait",
            ("play", "track7.ogg", 7500),
            "wait",
        ]
        
        self.missiles   = []
        
        self.screen = pygame.display.get_surface()
    
    def add_missile(self, missile):
        self.missiles.append(missile)
    
    def draw(self, (dx, dy)=(0, 0)):
        # Sorting missiles
        bg_missiles = [m for m in self.missiles if m.z < defs.background_z_center]
        ct_missiles = [m for m in self.missiles if defs.background_z_center <= m.z < defs.foreground_z_center]
        fg_missiles = [m for m in self.missiles if m.z >= defs.foreground_z_center]
        
        # Drawing the background items
        self.screen.blit(self.background, (dx, dy))
        for m in bg_missiles:
            m.draw(self.screen, (dx, dy))
        for e in self.enemies:
            e.draw(self.screen, (dx, dy))
        
        # Drawing the center items
        self.screen.blit(self.castle, (dx, dy))
        for m in ct_missiles:
            m.draw(self.screen, (dx, dy))
        self.player.draw(self.screen, (dx, dy))
        
        # Drawing the foreground items
        for m in fg_missiles:
            m.draw(self.screen, (dx, dy))
    
    def update_enemy_queue(self, delay):
        # Enemy queue handling
        if len(self.enemy_queue) == 0:
            return
        
        if self.enemy_queue[0][0] == "play":
            (crap, sound_file, crap2) = self.enemy_queue.pop(0)
            if pygame.mixer.get_init():
                sound = pygame.mixer.Sound(join("data", "sound", sound_file))
                sound.play()

        if len(self.enemy_queue) == 0:
            return

        if self.enemy_queue[0] == "wait":
            if pygame.mixer.get_init():
                if pygame.mixer.get_busy():
                    return
            if len(self.enemies) > 0:
                return
            else:
                self.enemy_queue.pop(0)
        
        if len(self.enemy_queue) == 0:
            return

        self.enemy_timer += delay
        
        # Setting up new enemies
        while len(self.enemy_queue) > 0 and self.enemy_timer > self.enemy_queue[0][1]:
            (new_enemy, crap) = self.enemy_queue.pop(0)
            new_enemy.set_state(defs.state_arrive)
            new_enemy.state_timer = new_enemy.sprite.current_animation.duration
            self.enemies.append(new_enemy)
        
    def update(self, delay, keys, events):
        self.update_enemy_queue(delay)
            
        # Updating missile positions
        for m in self.missiles:
            m.update(delay, events, self)
        
        # Updating actor positions
        actors = self.enemies + [self.player]
        for a in actors:
            a.think(keys, events, self)
            a.update(delay, events, keys, self)
        
        # Verifying collisions
        for m in self.missiles:
            if m.dz < 0:
                if defs.background_z_range[0] < m.z <= defs.background_z_range[1]:
                    for e in self.enemies:
                        if e.rect.collidepoint((m.x, m.y)):
                            m.delete_flag = True
                            e.hit(m)
            elif m.dz > 0:
                if defs.foreground_z_range[0] < m.z <= defs.foreground_z_range[1]:
                    if self.player.rect.collidepoint((m.x, m.y)):
                        m.delete_flag = True
                        if self.player.state == defs.state_catch:
                            self.player.state_timer = 0
                            self.player.weapon = m
                        else:
                            self.player.hit(m)
        
        #Deleting the killed actors and missiles that are done with their trajectory
        self.missiles = [m for m in self.missiles if not m.can_delete()]
        self.enemies  = [e for e in self.enemies  if not e.can_delete()]
